% Free-vibration analysis of the wet modes of us structure vibrating in
% contact with an infinite extent of incompressible fluid. The boundary
% element method is used to solve the Laplace equation for the pressure
% modes which are then used to create the added-mass matrix.
% 
% function  [W,Omega,psis] =wet_free_vibration_bem_solver(fens,bdry_gcells,W,Omega,rhow,u)
%
% fens = array of finite element nodes (all the nodes involved in the dry
% Vibration analysis),
% bdry_gcells =
% W = array of dry vibration modes,
% Omega= square matrix with the dry-vibration natural frequencies on the Diagonal,
% rhow= the mass density of the surrounding fluid,
% u= array of displacements used to construct the stiffness/mass matrices
% (this is used only for the equation numbers)
% 
% Returns:
% W= array of wet vibration modes,
% Omega= square matrix with the dry-vibration natural frequencies on the Diagonal,
% psis=Array of pressure modes
% 
function  [W,Omega,psis] =wet_free_vibration_bem_solver(fens,bdry_gcells,W,Omega,rhow,u)
    [nrmls,areas]=nodal_normals_areas(fens,bdry_gcells);
    Wns=nodal_normal_component(nrmls,W,u);
    psis=solve_pressure_modes(fens,bdry_gcells,Wns);
    neigvs =length(diag(Omega));
    rK =Omega;
    rM =eye (neigvs);
    raM =zeros(neigvs);
    for i=1:neigvs
        for j=1:neigvs
            raM(i,j) =rhow*(Wns(:,i).*areas)'*psis(:,j);
        end
    end
    raM = (raM +raM')/2;% it should be symmetric, but make sure

    [a,Omega]=eig(rK,rM+raM);
    W=W*a;
end

% Compute the pressure modes by solving the Laplace equation in an infinite
% domain, with Neumann boundary condition on the wet boundary, the gcells
% bdry_gcells. The slope of the pressure modes on the wet boundary is given
% by the normal component of the dry free-vibration mode, the array of
% fields Wns (one for each mode).
% The resulting pressure modes are returned as an array.
function psis=solve_pressure_modes(fens,bdry_gcells,Wns)
    if (~strcmp(class(bdry_gcells),'gcellset_Q4'))
        error (['The solver handles only boundary gcells of type gcellset_Q4']);
    end
    disp( ['Solver setup'])
    [nrmls,areas,gxyz]=normals_areas_centers(fens,bdry_gcells);
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    nxyz = gather(geom, (1:count(fens)), 'values', 'noreshape');
    f2gmap=fenode_to_gcell_map (struct ('gcells',bdry_gcells));
    gmap=get (f2gmap,'gcell_map');
    conns =get(bdry_gcells,'conn');
    clear j

    % This is a major cost factor, one-point quadrature may be good enough
    % except for thinner structures with Wet boundary faces from both
    % sides: then the 2 x 2 rule may be preferable
    integration_rule =gauss_rule(2,1);
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell=get(integration_rule, 'npts');

    clear N Nder
    for j=1:npts_per_gcell
        N{j} = bfun(bdry_gcells,pc(j,:));
        Nder{j} = bfundpar (bdry_gcells, pc(j,:));
    end
    clear j

    K =size(conns,1);
    M =size(Wns,2);

    W_n =zeros(K,M);
    for m=1:M
        for j=1:size(conns,1)
            W_n(j,m) =sum(Wns(conns(j,:),m))/4;
        end
    end

    disp( ['A(' num2str(K) ',' num2str(K) '),b(' num2str(K) ',' num2str(M) ') matrix'])

    A =zeros(K,K);
    b =zeros(K,M);
    for k=1:size(conns,1)
        if (mod(k,100)==0), disp( ['Row ' num2str(k)]); end
        A(k,k) =-1/2;
        xk=gxyz(k,:);
        for m=1:size(conns,1)
            xm=gxyz(m,:);
            if (m~=k)
                [Akm,Bkm] = integrate_coefficients(nxyz(conns(m,:),:),xk,nrmls(m,:),areas(m));
                A(k,m) = -Akm;
                b(k,:) = b(k,:) -Bkm*W_n(m,:);
            else
                %                                 Bkk = integrateBkk(nxyz(conns(m,:),:),xk,20);
                %                 1/(4*pi)*2*pi*sqrt(pi/4*Area_m/pi)*W_n(m,:);
                %                 1/4*(sqrt(Area_m/4)+sqrt(sqrt(2)*Area_m/4))*W_n(m,:);
                Area_k = areas(k);
                Bkk = 1/4*(sqrt(Area_k/4)+sqrt(2*Area_k/4));
                b(k,:) =b(k,:) -Bkk*W_n(k,:);
            end
        end
    end

    sol=A\b;
    clear A; clear b;


    % Define the Pressure mode field
    psis =zeros(count(fens),M);
    for j=1:size(conns,1)
        for f=1:M
            psis(conns(j,:),f) =psis(conns(j,:),f) +sol(j,f)/4;
        end
    end
    return;

    function [Akm,Bkm] = integrate_coefficients(gx,x0,n, area)
        detJ = area/4;
        Akm = 0;Bkm = 0;
        for pt=1:npts_per_gcell
            x=N{pt}'*gx;
            %             tangents =gx'*Nder{pt};
            %             Normal =[-tangents(3,1)*tangents(2,2)+tangents(2,1)*tangents(3,2); ...
            %             tangents(3,1)*tangents(1,2)-tangents(1,1)*tangents(3,2); ...
            %             -tangents(2,1)*tangents(1,2)+tangents(1,1)*tangents(2,2)];
            %             detJ = norm(Normal);
            r=(x-x0);  nr = norm(r);
            Akm =Akm +w(pt)*detJ*Gn(nr,n*r');
            Bkm =Bkm +w(pt)*detJ*G(nr);
        end
    end
    function Bkk = integrateBkk(gx,x0,nqp)
        %         nqp = 3;
        pc1 =linspace(-2+1/nqp,2- 1/nqp,nqp);
        w1 = 4/(nqp*nqp);
        Bkk = 0;
        for J1=1:nqp
            for K1=1:nqp
                N1 = bfun(bdry_gcells,[pc1(J1),pc1(K1)]);
                Nder1 = bfundpar (bdry_gcells,[pc1(J1),pc1(K1)]);
                x=N1'*gx;
                tangents =gx'*Nder1;
                Normal =skewmat(tangents(:,1))*tangents(:,2);
                detJ = norm(Normal);
                Bkk =Bkk +w1*detJ*G(x,x0);
            end
        end
    end
    function [nrmls,areas,xyz]=normals_areas_centers(fens,bdry_gcells);
        geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
        conns = get(bdry_gcells, 'conn'); % connectivity
        Nder = bfundpar (bdry_gcells, [0, 0]);

        nrmls =zeros(size(conns,1),3);
        areas =zeros(size(conns,1),1);
        xyz =zeros(size(conns,1),3);
        for j1=1:size(conns,1)
            conn = conns(j1,:); % connectivity
            kx = gather(geom, conn, 'values', 'noreshape');
            tangents =kx'*Nder;
            Normal =skewmat(tangents(:,1))*tangents(:,2);
            detJ = norm(Normal);
            nrmls(j1,:) =Normal'/detJ;
            areas(j1) = 4*detJ;
            xyz(j1,:) =sum(kx)/4;
        end
    end
    function g = G(nr)%(x,x0)
        %
        % 3-D free-field Green's function for the Laplace equation
        % x    is the position vector of the observation point
        % x0    is the position vector of the source (same size as x0)
%         g = 1/(4*pi)/(norm(x-x0));
        g = 0.079577471545948/(nr);
    end
    function g = Gn(nr,nxx0)%(x,x0,n)
        %
        % Normal derivative of the 3-D free-field Green's function for the Laplace equation
        %
        % x    is the position vector of the observation point
        % x0    is the position vector of the source (same size as x0)
%         r=(x-x0);
%         g = 1/(4*pi)*((n*r')/(norm(r)^3));
        g = 0.079577471545948*((nxx0)/(nr^3));
    end
end



% Compute the normal component of a free-vibration mode. The result is
% returned as a field.  Outer normal is assumed. The normals array is
% computed by nodal_normals_areas().
function Wns=nodal_normal_component(nrmls,W,u)
    M=size(W,2);
    Wns =zeros(size(nrmls,1),M);
    for i=1:M
        wv = get(scatter_sysvec(0*u, W(:,i)),'values');
        for j=1:size(nrmls,1)
            Wns(j,i) = nrmls(j,:)*wv(j,:)';
        end
    end
end


% Compute the Outer solid normals For all the nodes connected to the
% boundary gcells. The result is
% returned as an array, one normal per row.
function [nrmls,areas]=nodal_normals_areas(fens,bdry_gcells)
    conns =get(bdry_gcells,'conn');
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    xyz = gather(geom, (1:count(fens)), 'values', 'noreshape');
    Nder = bfundpar (bdry_gcells, [0, 0]);

    nrmls =0* xyz;
    areas =0* xyz(:,1);
    for j=1:size(conns,1)
        conn = conns(j,:);
        kx = gather(geom, conn, 'values', 'noreshape');
        tangents =kx'*Nder;
        Normal =skewmat(tangents(:,1))*tangents(:,2);
        detJ = norm(Normal);
        for k=1:4
            nrmls(conns(j,k),:) =nrmls(conns(j,k),:)+ Normal';
            areas(conns(j,k)) =areas(conns(j,k))+detJ;
        end
    end
    for j=1:size(nrmls,1)
        nn =norm(nrmls(j,:));
        if (nn~=0)
            nrmls(j,:) =nrmls(j,:)/nn;
        end
    end
end
