% Compute the element load vectors.
%
% function evs = distrib_loads(self, geom, u, fi, m)
%
% Compute the element load vectors corresponding to applied distributed
% load per unit volume in the reference, undeformed configuration. Here it
% means force per unit volume where volume could be length^3, length^2,
% length^1, or length^0, depending on the manifold dimension of the
% geometric cells.
% 
% Return an array of the load vectors so they may be assembled.
%    Call as
% evs = distrib_loads(feb, geom, u, fi, m)
%     geom=geometry field
%     u=displacement field
%     fi=force intensity object
%     m= manifold dimension
%
function evs = distrib_loads(self, geom, u, fi, m)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    dim = get(u,'dim');
    nfens=get(gcells,'nfens');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    %     Preallocate
    Fes= cell(ngcells,1);
    eqnums = cell(ngcells,1);
    for i=1:ngcells
        eqnums{i} =[];
        Fes{i} = [];
    end
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x = Xs(conn,:); % node coord
        % Loop over all integration points
        Fe = zeros(dim*nfens, 1); % element vector
        for j=1:npts_per_gcell
            N = bfun (gcells, pc(j,:));
            Nder = bfundpar(gcells, pc(j,:));
            J = Jacobian_matrix (gcells, Nder, x);
            detJ = Jacobian_mdim(gcells,conn,N,J,x,m); %Jacobian_mdim(self, conn, N, J, x, m)
            f=get_magn(fi,N'*x);
            Fe = Fe + reshape(f*N',dim*nfens,1) * (detJ * w(j));
        end
        Fes{i} = Fe;
        eqnums{i} =  gather(u, conn, 'eqnums');
    end
    evs  = elevecset(struct('vec',{Fes(1:end)}, 'eqnums',{eqnums(1:end)}));
    return;
end
