% Inspect the integration point quantities.
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
% Input arguments
%     x        - reference geometry field
%     u        - displacement field
%     dT       - temperature difference field
%     output   - this is what you would assign to the output field all of
%                the context argument of the material update() method.
%     component- component of the output array: see the material update()
%                method.
% Output argument
%     fld - the new field that can be used to map values the colors
%
function fld = field_from_integration_points(self, x, u, dT, output, component)
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(x,'dim');
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts=get(integration_rule, 'npts');
    nconn =length(get(gcells(1), 'conn'));
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    context.F= [];
    % Make the inverse map from finite element nodes to geometric cells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    fenodemap=get (f2gmap,'fenodes');
    % Now loop over all gcells in the block
    fld =slice(x,1,'fld');
    nvals = gather (fld, (1: get(fld,'nfens')),'values');
    context.output = output;
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=fenodemap{nix};
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            lnmap =0*(1:length(gmap));
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                for j= 1:length(kconn)
                    if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                        lnmap(kconn(j))=kconn(j);
                    end
                end
            end
            patchconn=lnmap(lnmap>0);
            % now store the local numbers
            for k=1:length(patchconn)
                lnmap(patchconn(k)) =k;
            end
            lnmap(thisnn) =length(patchconn)+1;
            c = gather(x, thisnn, 'values', 'noreshape'); % coordinates of central node
            % material tangent
            Rm = material_directions(self,gcells(gl(1)),[],c);
            % preallocate the strain-displacement matrix
            Nspatialderavg=zeros((length(patchconn)+1),dim);
            Vpatch =0;
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                lx = gather(x, kconn, 'values', 'noreshape'); % coordinates of nodes
                lx=lx-ones(length(kconn),1)*c;
                % get the parametric coordinates
                pci=find(kconn == thisnn);% at which node in the element are we?
                thisnn_pc=pc(pci,:); % parametric coordinates of that node
                Nder = bfundpar(gcells(gl(k)), thisnn_pc);
                xl=lx*Rm;% local coordinates
                Nspatialder = bfundsp (gcells(gl(k)), Nder, xl);
                detJ = Jacobian_volume(gcells(gl(k)), thisnn_pc,xl);
                Ve = detJ*w(pci);
                Vpatch=Vpatch+Ve;
                Nspatialderavg(lnmap(kconn),:)=Nspatialderavg(lnmap(kconn),:)+Ve*Nspatialder;
            end
            Nspatialderavg=Nspatialderavg/Vpatch;
            conn= [patchconn thisnn];
            Xnod=gather(x,[patchconn thisnn],'values','noreshape');
            Unod=gather(u,[patchconn thisnn],'values','noreshape');
            xnod=Xnod+Unod;
            Fnodal= xnod'*Nspatialderavg;
            context.F = Fnodal;
            if isempty(dT)
                dTs=zeros(length(conn),1);
            else
                dTs = gather(context.dT, conn, 'values', 'noreshape'); % displacement
            end
            context.dT = dTs(end); % temperature at the node
            context.xyz =c;
            [out,ignore] = update(mat, matstates{nix,1}, context);
            nvals (nix) = out(component);
        end
    end
    fld = scatter(fld, (1: get(fld,'nfens')),nvals);
    return;
end