% Compute the stabilization restoring force vectors
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
% 
% function [self,evs] = restoring_force(self, geom, u1, u, stabfact)
% 
%    Call as
% evs = restoring_force(feb, geom, u1, u, stabfact), or
% [feb,evs] = restoring_force(feb, geom, u1, u, stabfact)
%     geom     - geometry field
%     u1       - displacement field at the end of time step t_n+1
%     u        - displacement field at the end of time step t_n
%     stabfact - stabilization coefficient, some fraction of the representative
%               elastic moduli
%
% Note: This method *UPDATES* the state of the feblock object.  In
%       particular, the material state gets updated.  If this gets
%       called for converged u1, the feblock must be assigned to itself
%       on return; otherwise the first return value must be ignored!
%
function [self,evs] = restoring_force(self, geom, u1, u, stabfact)
    if (~exist('stabfact','var')), stabfact =1; end
    if (stabfact==0),
        evs=[]; return;
    end
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    %npts_per_gcell = length(pc); % number of integration point
    npts=get(integration_rule, 'npts');
    % Material
    mat = get(self, 'mater');
    matstates = get(self, 'matstates');
    % Make the inverse map from finite element nodes to geometric cells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    fenodemap=get (f2gmap,'fenodes');
    % Pre-allocate the element matrices
    i=0;
    for k = 1:length(gmap)
        gl = gmap{k};
        i = i + length(gl);
    end
    evs(1:i) = deal(elevec);
    iem=1;
    % Now loop over all finite element nodes
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=fenodemap{nix};
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            lnmap =0*(1:length(gmap));
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                for j= 1:length(kconn)
                    if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                        lnmap(kconn(j))=kconn(j);
                    end
                end
            end
            patchconn=lnmap(lnmap>0);
            % now store the local numbers
            for k=1:length(patchconn)
                lnmap(patchconn(k)) =k;
            end
            lnmap(thisnn) =length(patchconn)+1;
            c = gather(geom, thisnn, 'values', 'noreshape'); % reference coordinates of central node
            Rm = material_directions(self,gcells(gl(1)),[],c);
            % preallocate the strain-displacement matrix
            Nspatialderavg=zeros((length(patchconn)+1),dim);
            Vpatch =0;
			for k=1:length(gl)
				kconn = get(gcells(gl(k)), 'conn'); % connectivity
        		X = gather(geom, kconn, 'values', 'noreshape'); % reference coordinates
				%U = gather(u, kconn, 'values', 'noreshape'); % displacement in step n
				U1 = gather(u1, kconn, 'values', 'noreshape'); % displacement in step n+1
				%x = X + U; % previous known coordinates
				x1 = X + U1; % current coordinates
				pci=find(kconn == thisnn);% at which node in the element are we?
                if (pci<=npts)
                    thisnn_pc=pc(pci,:); % parametric coordinates of that node
%                     Nder = bfundpar(gcells(gl(k)), thisnn_pc);
                    Nder = self.Nder{pci};
                    NderX=bfundsp(gcells(gl(k)),Nder,X*Rm);
                    detJ = Jacobian_volume(gcells(gl(k)), thisnn_pc,X);
                    Ve = detJ*w(pci);
                    Vpatch=Vpatch+Ve;
                    Nspatialderavg(lnmap(kconn),:)= Nspatialderavg(lnmap(kconn),:)+ Ve*NderX;
                end
            end
            Nspatialderavg=Nspatialderavg/ Vpatch;
%             if (self.overlay) Vpatch =Vpatch/2; end
            Gnodal= self.hGmat (self,Nspatialderavg);
            Xnod=gather(geom,[patchconn thisnn],'values');
            U1nod=gather(u1,[patchconn thisnn],'values');
            x1nod=Xnod+U1nod;
            Fe=[];
            for k=1:length(gl)
				kconn = get(gcells(gl(k)), 'conn'); % connectivity
        		X = gather(geom, kconn, 'values', 'noreshape'); % reference coordinates
				pci=find(kconn == thisnn);% at which node in the element are we?
                if (pci<=npts)
                    thisnn_pc=pc(pci,:); % parametric coordinates of that node
                    NderX=bfundsp(gcells(gl(k)),self.Nder{pci},X*Rm);
                    detJ = Jacobian_volume(gcells(gl(k)), thisnn_pc,X);
                    Ve = detJ*w(pci);
                    Nspatialder=zeros((length(patchconn)+1),dim);
                    Nspatialder(lnmap(kconn),:)= NderX;
                    Gele= self.hGmat (self,Nspatialder);
                    G=Gnodal-Gele;
                    if (isempty(Fe)), Fe = 0*(G'*  (G* x1nod)) * (stabfact * Ve); end
                    Fe = Fe -(G'*  (G* x1nod)) * (stabfact * Ve); % note the sign
                end
            end
            evs(iem) = set(evs(iem), 'vec', Fe);
            evs(iem) = set(evs(iem), 'eqnums', gather(u1, [patchconn thisnn], 'eqnums'));
            iem=iem+1;
        end
    end
    self.matstates = matstates;
    return;
end
