% Compute the material stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% function ems = stiffness (self, geom, u1, u, stabfact)
%     geom=reference geometry field
%     u1=displacement field in the *current* configuration
%     u=displacement field in the last known configuration
%     stabfact= stabilization coefficient, some fraction of the representative
%     elastic moduli
%    Returns an array of elemat objects that may be assembled
%    into the system stiffness matrix.
%
function ems = stiffness (self, geom, u1, u, stabfact)
    if (~exist('stabfact','var')), stabfact =1; end
    if (stabfact==0),
        ems=[]; return;
    end
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    %npts_per_gcell = length(pc);   % number of integration point
	npts=get(integration_rule, 'npts');
	%nconn =length(get(gcells(1), 'conn'));
    mat = get(self, 'mater'); % Material
    matstates = get(self, 'matstates');
    % Make the inverse map from finite element nodes to geometric cells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    fenodemap=get (f2gmap,'fenodes');
    % Pre-allocate the element matrices
    ems(1:length(gmap)) = deal(elemat);
    iem=1;
    % Now loop over all finite element nodes
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=fenodemap{nix};
		if ~isempty(gl) % This node has a element patch in this block
			% establish local numbering of all nodes of the patch @node nix
			lnmap =0*(1:length(gmap));
            for k=1:length(gl)
               	kconn = get(gcells(gl(k)), 'conn'); % connectivity
                for j= 1:length(kconn)
                    if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                      lnmap(kconn(j))=kconn(j);
                    end
                end
            end
            patchconn=lnmap(lnmap>0);
            % now store the local numbers
            for k=1:length(patchconn)
                lnmap(patchconn(k)) =k;
            end
            lnmap(thisnn) =length(patchconn)+1;
			c = gather(geom, thisnn, 'values', 'noreshape'); % reference coordinates of central node
			Rm = material_directions(self,gcells(gl(1)),[],c);
            % preallocate the strain-displacement matrix
			Nspatialderavg=zeros((length(patchconn)+1),dim);
			Vpatch =0;
			for k=1:length(gl)
				kconn = get(gcells(gl(k)), 'conn'); % connectivity
        		X = gather(geom, kconn, 'values', 'noreshape'); % reference coordinates
				U1 = gather(u1, kconn, 'values', 'noreshape'); % displacement in step n+1
				x1 = X + U1; % current coordinates
				pci=find(kconn == thisnn);% at which node in the element are we?
                if (pci<=npts)
                    thisnn_pc=pc(pci,:); % parametric coordinates of that node
%                     Nder = bfundpar(gcells(gl(k)), thisnn_pc);
                    Nder = self.Nder{pci};
                    NderX=bfundsp(gcells(gl(k)),Nder,X*Rm);
                    detJ = Jacobian_volume(gcells(gl(k)), thisnn_pc,X);
                    Ve = detJ*w(pci);
                    Vpatch=Vpatch+Ve;
                    Nspatialderavg(lnmap(kconn),:)= Nspatialderavg(lnmap(kconn),:)+ Ve*NderX;
                end
            end
            Nspatialderavg=Nspatialderavg/ Vpatch;
%             if (self.overlay) Vpatch =Vpatch/2; end
            Gnodal= self.hGmat (self,Nspatialderavg);
            Ke=[];
            for k=1:length(gl)
				kconn = get(gcells(gl(k)), 'conn'); % connectivity
        		X = gather(geom, kconn, 'values', 'noreshape'); % reference coordinates
				pci=find(kconn == thisnn);% at which node in the element are we?
                if (pci<=npts)
                    thisnn_pc=pc(pci,:); % parametric coordinates of that node
%                     Nder = bfundpar(gcells(gl(k)), thisnn_pc);
                    Nder = self.Nder{pci};
%                     Rm = material_directions(self,gcells(gl(k)),thisnn_pc,X);
					NderX=bfundsp(gcells(gl(k)),Nder,X*Rm);
                    detJ = Jacobian_volume(gcells(gl(k)), thisnn_pc,X);
                    Ve = detJ*w(pci);
                    Nspatialder=zeros((length(patchconn)+1),dim);
                    Nspatialder(lnmap(kconn),:)= NderX;
                    Gele= self.hGmat (self,Nspatialder);
                    G=Gnodal-Gele;
                    if (isempty(Ke)), Ke = 0*G'* G; end
                    Ke = Ke+ stabfact * G'* G*  Ve; 
                end
            end
            ems(iem) = set(ems(iem), 'mat', Ke);
            ems(iem) = set(ems(iem), 'eqnums',gather(u1, [patchconn thisnn], 'eqnums'));
            iem=iem+1;
        end
    end
    return;
end
