% Compute the local geometric stiffness matrix. 
%
% function SM = local_geometric_stiffness_2nd(self,A,I2,I3,PN,L)
% 
% Inputs:
% A= cross-sectional area, 
% I2, I3=central moment of inertia of the cross-section about the x2 and x3 
% coordinate axis, 
% PN= vector of natural forces; see natural_forces() for definitions
% L= current length of the element, 
% 
% Outputs:
% SM = local geometric stiffness matrix, 12 x 12
% 
% This geometric stiffness matrix is based on a consistent second-order
% formulation from
% @article{Argyris:1979,
%    author = {Argyris, J. H. and Hilpert, O. and Malejannakis, G. A. and Scharpf, D. W.},
%    title = {GEOMETRICAL STIFFNESS OF A BEAM IN SPACE - CONSISTENT VW APPROACH},
%    journal = {Computer Methods in Applied Mechanics and Engineering},
%    volume = {20},
%    number = {1},
%    pages = {105-131},
%    year = {1979}
% } 
function SM = local_geometric_stiffness_2nd(self,A,I2,I3,PN,L)
    N=PN(1);
    S_2=-2*PN(3)/L;
    S_3=-2*PN(5)/L;
    M_1=PN(6);
    M_2s=PN(4);
    M_3s=-PN(2);
    s11=[0,-S_2/L,-S_3/L;
        -S_2/L,6/5*N/L,0;
        -S_3/L,0,6/5*N/L];
    s12=[0,0,0;
        -(M_2s-L/2*S_3)/L,M_1/L,N/10;
        (-M_3s-L/2*S_2)/L,-N/10,M_1/L]; s21=s12';
    s13=[0,+S_2/L,+S_3/L;
        +S_2/L,-6/5*N/L,0;
        +S_3/L,0,-6/5*N/L]; s31=s13';
    s14=[0,0,0;
        (M_2s+L/2*S_3)/L,-M_1/L,N/10;
        -(-M_3s+L/2*S_2)/L,-N/10,-M_1/L]; s41=s14';
    s22=[(I2+I3)/A/L*N,+M_3s/2+L/2*S_2/6,-M_2s/2+L/2*S_3/6;
        +M_3s/2+L/2*S_2/6,2*L/15*N,0;
        -M_2s/2+L/2*S_3/6,0,2*L/15*N];
    s23=[0,(M_2s-L/2*S_3)/L,-(-M_3s-L/2*S_2)/L;
        0,-M_1/L,N/10;
        0,-N/10,-M_1/L]; s32=s23';
    s24=[-(I2+I3)/A/L*N,+L/2*S_2/3,+L/2*S_3/3;
        +L/2*S_2/3,-N*L/30,M_1/2;
        +L/2*S_3/3,-M_1/2,-N*L/30]; s42=s24';
    s33=[0,-S_2/L,-S_3/L;
        -S_2/L,6/5*N/L,0;
        -S_3/L,0,6/5*N/L];
    s34=[0,0,0;
        -(M_2s+L/2*S_3)/L,M_1/L,-N/10;
        (-M_3s+L/2*S_2)/L,N/10,M_1/L]; s43=s34';
    s44=[(I2+I3)/A/L*N,-M_3s/2+L/2*S_2/6,M_2s/2+L/2*S_3/6;
        -M_3s/2+L/2*S_2/6,2*L/15*N,0;
        M_2s/2+L/2*S_3/6,0,2*L/15*N];
    SM=[s11,s12,s13,s14;
        s21,s22,s23,s24;
        s31,s32,s33,s34;
        s41,s42,s43,s44;
        ];
end
% % s11=[0,2*PN(3)/L/L,2*PN(5)/L/L;
% %         2*PN(3)/L/L,6/5*PN(1)/L,0;
% %         2*PN(5)/L/L,0,6/5*PN(1)/L];
% %     s12=[0,0,0;
% %         -(PN(4)+PN(5))/L,PN(6)/L,PN(1)/10;
% %         (PN(2)+PN(3))/L,-PN(1)/10,PN(6)/L]; s21=s12';
% %     s13=[0,-2*PN(3)/L/L,-2*PN(5)/L/L;
% %         -2*PN(3)/L/L,-6/5*PN(1)/L,0;
% %         -2*PN(5)/L/L,0,-6/5*PN(1)/L]; s31=s13';
% %     s14=[0,0,0;
% %         (PN(4)-PN(5))/L,-PN(6)/L,PN(1)/10;
% %         -(PN(2)-PN(3))/L,-PN(1)/10,-PN(6)/L]; s41=s14';
% %     s22=[(I2+I3)/A/L*PN(1),-PN(2)/2-PN(3)/6,-PN(4)/2-PN(5)/6;
% %         -PN(2)/2-PN(3)/6,2*L/15*PN(1),0;
% %         -PN(4)/2-PN(5)/6,0,2*L/15*PN(1)];
% %     s23=[0,(PN(4)+PN(5))/L,-(PN(2)+PN(3))/L;
% %         0,-PN(6)/L,PN(1)/10;
% %         0,-PN(1)/10,-PN(6)/L]; s32=s23';
% %     s24=[-(I2+I3)/A/L*PN(1),-PN(3)/3,-PN(5)/3;
% %         -PN(3)/3,-PN(1)*L/30,PN(6)/2;
% %         -PN(5)/3,-PN(6)/2,-PN(1)*L/30]; s42=s24';
% %     s33=[0,2*PN(3)/L/L,2*PN(5)/L/L;
% %         2*PN(3)/L/L,6/5*PN(1)/L,0;
% %         2*PN(5)/L/L,0,6/5*PN(1)/L];
% %     s34=[0,0,0;
% %         -(PN(4)-PN(5))/L,PN(6)/L,-PN(1)/10;
% %         (PN(2)-PN(3))/L,PN(1)/10,PN(6)/L]; s43=s34';
% %     s44=[(I2+I3)/A/L*PN(1),PN(2)/2-PN(3)/6,PN(4)/2-PN(5)/6;
% %         PN(2)/2-PN(3)/6,2*L/15*PN(1),0;
% %         PN(4)/2-PN(5)/6,0,2*L/15*PN(1)];
% %     SM=[s11,s12,s13,s14;
% %         s21,s22,s23,s24;
% %         s31,s32,s33,s34;
% %         s41,s42,s43,s44;
% %         ];