% Compute the stiffness matrices of the individual entities in the block
% (in this case nodes).
% Return  element matrix set so they may be assembled.
%    Call as
% function ems = stiffness (self, geom, u)
%     geom=geometry field
%     u=displacement field
%
function ems = stiffness (self, geom, u)
    gcells = get(self.feblock,'gcells');
    dim = get(geom,'dim');
    tangents =eye(dim);
    % Material
    mat = get(self.feblock, 'mater');
    %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    %     Preallocate
    Kes= cell(length(bfun_gradients),1);
    eqnums = cell(length(bfun_gradients),1);
    for i=1:length(bfun_gradients)
        eqnums{i} =[];
        Kes{i} = [];
    end
    xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
    eqnumss =gather(u, 1:length(bfun_gradients), 'eqnums','noreshape');% equation numbers for all nodes
    % Now loop over all finite element nodes
    iem =1;
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = xs(nix, :); % coordinates of central node
            Dv = tangent_moduli (mat,struct('kind',self.split));
            Ds = tangent_moduli (mat, ...
                struct('kind',[self.split '_shear']));
            Bnodal= self.hBlmat (self,bfun_gradients{nix}.Nspd,c,[]);
            Ke = Bnodal'*(Dv+(1-self.stabfact)*Ds)*Bnodal * bfun_gradients{nix}.Vpatch;
            Kes{iem} =  Ke;
            eqnums{iem} =  reshape(eqnumss(bfun_gradients{nix}.patchconn,:)',(np)*size(eqnumss,2),1);
            iem=iem+1;
        else
            warning(['Patch volume = ' num2str(bfun_gradients{nix}.Vpatch)]);
        end
    end
    ems  = elematset(struct('mat',{Kes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
    % Should we do physical  stabilization?
    if (self.stabfact>0)
        gcells = get(self,'gcells');
        ngcells = count(gcells);
        nfens = get(gcells(1),'nfens');
        dim = get(geom,'dim');
        % Integration rule
        integration_rule = get(self, 'integration_rule');
        % - shear terms
        pc_s = get(integration_rule, 'param_coords');
        w_s  = get(integration_rule, 'weights');
        n_s = get(integration_rule, 'npts');
        for j=1:n_s
            Ns_s{j} = bfun(gcells,pc_s(j,:));
            Nders_s{j} = bfundpar(gcells,pc_s(j,:));
        end
        % Material
        mat = get(self, 'mater');
        % Loop over all the geometry cells
        conns = get(gcells, 'conn'); % connectivity
        eKes = cell(size(conns,1),1);
        eqnums = cell(size(conns,1),1);
        for i=1:size(conns,1)
            eqnums{i} =gather(u,conns(i,:),'eqnums');
            eKes{i} =zeros(get(geom,'dim')*nfens);
        end
        xs =get(geom,'values');
        for i=1:size(conns,1)
            conn =conns(i,:);
            x=xs(conn,:);
            for j=1:n_s
                c =Ns_s{j}'*x;% physical location
                J = x' * Nders_s{j};
                Jac = Jacobian_volume(gcells,conn, Ns_s{j}, J, x);
                if (Jac > 0)
                    Ndersp = Nders_s{j}/J;% derivatives wrt local coor
                    B = self.hBlmat (self,Ndersp,c,[]);% strain-displacement
                    Ds = tangent_moduli (mat, ...
                                         struct('kind',[self.split '_shear']));
                    eKes{i} = eKes{i} + (B'*(Ds*(self.stabfact*Jac*w_s(j)))*B);
                end
            end
        end
        eems  = elematset(struct('mat',{eKes}, 'eqnums',{eqnums}));
        ems=cat(2,ems,eems);
    end
end


