% restoring_force: Compute the restoring force vectors
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
%    Call as
% evs = restoring_force(feb, geom, u)
%     geom     - geometry field
%     u1       - displacement field at the end of time step t_n+1
%     u        - displacement field at the end of time step t_n
%
% Note: This method *UPDATES* the state of the feblock object.  In
%       particular, the material state gets updated.  If this gets
%       called for converged u1, the feblock must be assigned to itself
%       on return; otherwise the first return value must be ignored!
%
function [self,evs] = restoring_force(self, geom, u1, u)
    gcells = get(self.feblock_defor_ss,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock_defor_ss, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = length(pc); % number of integration point
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock_defor_ss, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom,'dim')*nfens,1);
    end
    Xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    Blmat =get(self,'Blmat');
    matstates = self.matstates;
    for i=1:ngcells
        conn =conns(i,:);
        X=Xs(conn,:);
        U1 = gather(u1, conn, 'values');  % displacements of nodes
        context.dT = 0;
        for j=1:npts_per_gcell
            c =X'*self.N{j};% physical location
            J = X' * self.Nder{j};%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, self.N{j}, J, X);
            if (isempty(Rm))
                Ndersp = self.Nder{j}/J;% derivatives wrt local coor
            else
                Ndersp = self.Nder{j}/(Rm'*J);% derivatives wrt local coor
            end
            B = Blmat (self, self.N{j},Ndersp, c, Rm);
            context.strain = B*U1;
            context.c=c;
            [cauchy,matstates{i,j}] = update(mat, matstates{i,j}, context);% cauchy is Cauchy's stress tensor
            Fe{i} = Fe{i} - B'* (cauchy * Jac * w(j)); % note the sign
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
    self.matstates = matstates;
    return;
end