% Constructor of the geometric cell set class.
%
% function retobj = gcellset(varargin)
%
% Constructor of the class that represents a geometric cell, which encloses some part
% of the computational domain.
% This class is the base class for gcells, i.e. every usable gcell has to be
% derived from it.
% 
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%     id=identifier of this gcell: any integer.  This is useful
%        for debugging.
%     conn=connectivity of the gcell: numbers of nodes this cell connects.
%     nfens=number of nodes this cell *should* connect; it is checked
%           against length(conn): if there is a mismatch, an error is reported.
%  optional
%     label=numerical label, supplied for each cell in the set, or 
%           a single number to be applied to all cells
function retobj = gcellset(varargin)
    class_name='gcellset';
    if (nargin == 0)
        parent=classbase;
        retobj.id = 0;
        retobj.conn = [];
        retobj.nfens = 0;
        retobj.label = [];
        retobj = class(retobj,class_name,parent);
        return;
    elseif (nargin == 1)
        parent=classbase;
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            retobj.id = [];
            if isfield(Parameters,'id')
                retobj.id = Parameters.id;
            end
            retobj.conn = [];
            if isfield(Parameters,'conn')
                retobj.conn = Parameters.conn;
            end
            retobj.nfens = 0;
            if isfield(Parameters,'nfens')
                retobj.nfens = Parameters.nfens;
            end
            retobj.label = [];
            if isfield(Parameters,'label')
                retobj.label = Parameters.label;
                if (length(retobj.label)==1)
                    retobj.label=zeros(size(retobj.conn,1),1)+Parameters.label;
                end
            end
            if (~ isempty(retobj.conn) & (retobj.nfens ~= size(retobj.conn,2)))
                error('Wrong number of nodes!');
            end
            retobj = class(retobj,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end

